# -*- coding: utf-8 -*-
from typing import Optional

from pip_services4_commons.commands import CommandSet, ICommand, Command
from pip_services4_commons.convert import TypeCode
from pip_services4_commons.data import FilterParams, PagingParams
from pip_services4_commons.run import Parameters
from pip_services4_commons.validate import ObjectSchema, FilterParamsSchema, PagingParamsSchema

from ..data import EntityV1
from ..data import EntityV1Schema
from .IBasicService import IBasicService


class BasicCommandSet(CommandSet):

    def __init__(self, service: IBasicService):
        super().__init__()

        self.__service = service

        self.add_command(self.__make_get_entities_command())
        self.add_command(self.__make_get_entity_by_id_command())
        self.add_command(self.__make_get_entity_by_name_command())
        self.add_command(self.__make_create_entity_command())
        self.add_command(self.__make_update_entity_command())
        self.add_command(self.__make_delete_entity_by_id_command())

    def __make_get_entities_command(self) -> ICommand:
        def action(correlation_id: Optional[str], args: Parameters):
            filter = FilterParams.from_value(args.get('filter'))
            paging = PagingParams.from_value(args.get('paging'))
            return self.__service.get_entities(correlation_id, filter, paging)

        return Command(
            'get_entities',
            ObjectSchema(True)
                .with_optional_property('filter', FilterParamsSchema())
                .with_optional_property('paging', PagingParamsSchema()),
            action
        )

    def __make_get_entity_by_id_command(self) -> ICommand:
        def action(correlation_id: Optional[str], args: Parameters):
            entity_id = args.get_as_string('entity_id')
            return self.__service.get_entity_by_id(correlation_id, entity_id)

        return Command(
            'get_entity_by_id',
            ObjectSchema(True).with_optional_property('entity_id', TypeCode.String),
            action
        )

    def __make_get_entity_by_name_command(self) -> ICommand:
        def action(correlation_id: Optional[str], args: Parameters):
            name = args.get_as_string('name')
            return self.__service.get_entity_by_name(correlation_id, name)

        return Command(
            'get_entity_by_name',
            ObjectSchema(True).with_optional_property('name', TypeCode.String),
            action
        )

    def __make_create_entity_command(self) -> ICommand:
        def action(correlation_id: Optional[str], args: Parameters):
            entity = args.get_as_object('entity')
            if isinstance(entity, dict):
                entity = EntityV1(**entity)
            return self.__service.create_entity(correlation_id, entity)

        return Command(
            'create_entity',
            ObjectSchema(True).with_required_property('entity', EntityV1Schema()),
            action
        )

    def __make_update_entity_command(self) -> ICommand:
        def action(correlation_id: Optional[str], args: Parameters):
            entity = args.get_as_object('entity')
            if isinstance(entity, dict):
                entity = EntityV1(**entity)
            return self.__service.update_entity(correlation_id, entity)

        return Command(
            'update_entity',
            ObjectSchema(True).with_required_property('entity', EntityV1Schema()),
            action
        )

    def __make_delete_entity_by_id_command(self) -> ICommand:
        def action(correlation_id: Optional[str], args: Parameters):
            entity_id = args.get_as_string('entity_id')
            return self.__service.delete_entity_by_id(correlation_id, entity_id)

        return Command(
            'delete_entity_by_id',
            ObjectSchema(True).with_required_property('entity_id', TypeCode.String),
            action
        )



# -*- coding: utf-8 -*-
from copy import deepcopy
from typing import Optional, Callable, List

from pip_services4_commons.commands import ICommandable, CommandSet
from pip_services4_commons.config import IConfigurable, ConfigParams
from pip_services4_commons.data import FilterParams, PagingParams, DataPage, IdGenerator
from pip_services4_commons.refer import IReferenceable, Descriptor, IReferences

from ..data import EntityTypeV1
from ..data import EntityV1

from .BasicCommandSet import BasicCommandSet
from .IBasicService import IBasicService


class BasicService(IBasicService, IConfigurable, IReferenceable, ICommandable):

    def __init__(self):
        self.__max_page_size: int = 100
        self.__items: List[EntityV1] = []
        self.__command_set: BasicCommandSet = None

    def configure(self, config: ConfigParams):
        pass

    def set_references(self, references: IReferences):
        pass

    def __compose_filter(self, filter_params: FilterParams) -> Callable[[EntityV1], bool]:
        filter_params = filter_params or FilterParams()

        id = filter_params.get_as_nullable_string("id")
        siteId = filter_params.get_as_nullable_string("site_id")
        name = filter_params.get_as_nullable_string("name")

        temp_names = filter_params.get_as_nullable_string("names")
        names = None if temp_names is None else temp_names.split(",")

        def compare(item: EntityV1) -> bool:
            if id is not None and item.id != id:
                return False
            if siteId is not None and item.site_id is not siteId:
                return False
            if name is not None and item.name is not name:
                return False
            if names is not None and item.name in names:
                return False
            return True

        return compare

    def get_command_set(self) -> CommandSet:
        if self.__command_set is None:
            self.__command_set = BasicCommandSet(self)

        return self.__command_set

    def get_entities(self, correlation_id: Optional[str], filter_params: FilterParams,
                     paging: PagingParams) -> DataPage:
        filter_entities = self.__compose_filter(filter_params)
        entities = list(filter(filter_entities, self.__items))

        # Extract a page
        paging = paging or PagingParams()
        skip = paging.get_skip(-1)
        take = paging.get_take(self.__max_page_size)
        total = None

        if paging.total: total = len(entities)
        if skip > 0: entities = entities[skip:]

        entities = entities[:take]
        return DataPage(entities, total)

    def get_entity_by_id(self, correlation_id: Optional[str], entity_id: str) -> EntityV1:
        entities = list(filter(lambda x: x.id == entity_id, self.__items))
        entity = None if len(entities) == 0 else entities[0]
        return entity

    def get_entity_by_name(self, correlation_id: Optional[str], name: str) -> EntityV1:
        entities = list(filter(lambda x: x.name == name, self.__items))
        entity = None if len(entities) == 0 else entities[0]
        return entity

    def create_entity(self, correlation_id: Optional[str], entity: EntityV1) -> Optional[EntityV1]:
        if entity is None:
            return None

        entity.id = entity.id or IdGenerator.next_long()
        entity.type = entity.type or EntityTypeV1.Unknown

        self.__items.append(entity)

        return entity

    def update_entity(self, correlation_id: Optional[str], entity: EntityV1) -> Optional[EntityV1]:
        ids = list(map(lambda x: x.id, self.__items))

        if entity.id not in ids:
            return None

        index = ids.index(entity.id)

        entity = deepcopy(entity)
        self.__items[index] = entity

        return entity

    def delete_entity_by_id(self, correlation_id: Optional[str], entity_id: str) -> Optional[EntityV1]:
        ids = list(map(lambda x: x.id, self.__items))

        if entity_id not in ids:
            return None

        index = ids.index(entity_id)

        entity = self.__items[index]
        del self.__items[index]

        return entity



# -*- coding: utf-8 -*-

__all__ = ['IBasicController', 'BasicController', 'BasicCommandSet']

from .IBasicController import IBasicController
from .BasicController import BasicController
from .BasicCommandSet import BasicCommandSet



# -*- coding: utf-8 -*-
from abc import ABC, abstractmethod
from typing import Optional

from pip_services4_commons.data import FilterParams, PagingParams, DataPage

from ..data import EntityV1


class IBasicService(ABC):

    def get_entities(self, correlation_id: Optional[str], filter_params: FilterParams, paging: PagingParams) -> DataPage:
        raise NotImplementedError("Method is not implemented")

    def get_entity_by_id(self, correlation_id: Optional[str], entity_id: str) -> EntityV1:
        raise NotImplementedError("Method is not implemented")

    def get_entity_by_name(self, correlation_id: Optional[str], name: str) -> EntityV1:
        raise NotImplementedError("Method is not implemented")

    def create_entity(self, correlation_id: Optional[str], entity: EntityV1) -> EntityV1:
        raise NotImplementedError("Method is not implemented")

    def update_entity(self, correlation_id: Optional[str], entity: EntityV1) -> EntityV1:
        raise NotImplementedError("Method is not implemented")

    def delete_entity_by_id(self, correlation_id: Optional[str], entity_id: str) -> EntityV1:
        raise NotImplementedError("Method is not implemented")



# -*- coding: utf-8 -*-
from pip_services4_commons.config import ConfigParams
from pip_services4_commons.data import FilterParams, PagingParams
from pip_services4_commons.refer import References, Descriptor

from service_basic_pipservices.data import EntityTypeV1
from service_basic_pipservices.data import EntityV1
from service_basic_pipservices.logic import BasicService

ENTITY1 = EntityV1(id='1',
                   name='00001',
                   type=EntityTypeV1.Type1,
                   site_id='1',
                   content='ABC')

ENTITY2 = EntityV1(id='2',
                   name='00002',
                   type=EntityTypeV1.Type2,
                   site_id='1',
                   content='XYZ')


class TestBasicService:
    service: BasicService

    def setup_method(self):
        self.service = BasicService()
        self.service.configure(ConfigParams())

        references = References.from_tuples(
            Descriptor('service-basic', 'service', 'default', 'default', '1.0'), self.service
        )

        self.service.set_references(references)

    def test_crud_operations(self):
        # Create the first entity
        entity = self.service.create_entity(None, ENTITY1)
        assert ENTITY1.name == entity.name
        assert ENTITY1.site_id == entity.site_id
        assert ENTITY1.type == entity.type
        assert ENTITY1.name == entity.name
        assert entity.content is not None

        # Create the second entity
        entity = self.service.create_entity(None, ENTITY2)
        assert ENTITY2.name == entity.name
        assert ENTITY2.site_id == entity.site_id
        assert ENTITY2.type == entity.type
        assert ENTITY2.name == entity.name
        assert entity.content is not None

        # Get all entities
        page = self.service.get_entities(None, FilterParams(), PagingParams())
        assert page is not None
        assert len(page.data) == 2

        entity1: EntityV1 = page.data[0]

        # Update the entity
        entity1.name = 'ABC'

        entity = self.service.update_entity(None, entity1)
        assert entity1.id == entity.id
        assert 'ABC' == entity.name

        # Get entity by name
        entity = self.service.get_entity_by_name(None, entity1.name)
        assert entity1.id == entity.id

        # Delete the entity
        entity = self.service.delete_entity_by_id(None, entity1.id)
        assert entity1.id == entity.id

        # Try to get deleted entity
        entity = self.service.get_entity_by_id(None, entity1.id)
        assert entity is None
